#include "fpp.h"
#undef EXTERN

#define EXTERN
#include "hash.h"
#undef EXTERN

#include <errno.h>

#if mips
extern int errno;
#endif

#include <time.h>
#include <sys/types.h>

char		*Cuserid ARGS((char *s));

#if    unix
#include "xpwd.h"			/* for passwd  */
struct passwd	*passwdp;
#endif /* unix */

void
init(VOID_ARG)
{
    char date[12];			/* "Jul 24 1847" */
    int n;
    char *p;
    char timestring[26];		/* "Mon Dec 24 15:44:14 MST 1990" */

#if    OS_VMCMS
    struct tm			*thetimeval;
				/* clock value from time() for ctime() */
#else /* NOT OS_VMCMS */
    time_t			thetimeval;
				/* clock value from time() for ctime() */
#endif /* OS_VMCMS */

#if    unix
    char hostname[MAXLINE+1];
    const char *personal_name = "";
    const char *user_name;
#endif /* unix */

    errno = 0;
    fpp_code = 0;
    fpp_debug = 0;
    fpp_infile = (FILE*)NULL;
    level = 0;
    selected[0] = 1;
    branch_done[0] = 1;

    for (n = 1; n < MAXLEVEL; ++n)
    {
	selected[n] = 0;
	branch_done[n] = 0;
    }

    /* To reduce hash and malloc activity, we put the hash value of
       __LINE__ in global storage; it gets updated in getline() */

    n = install("__LINE__","");
    if (n >= 0)
    {
	free(hash_value[n]);
	hash_value[n] = fpp_number;
    }
    FNR = 0;
    NR = 0;

    PRINTF("%s %s\n",OUT_PREFIX,VERSION);

#if    OS_VMCMS
    thetimeval = time();
    (void)strcpy(timestring,ctime(thetimeval));
#else  /* NOT OS_VMCMS */
    thetimeval = time((time_t*)NULL);
    (void)strcpy(timestring,ctime(&thetimeval));
#endif /* OS_VMCMS */

    PRINTF("%s Date: %s",OUT_PREFIX,timestring);

    /* 000000000011111111112222 2 2 */
    /* 012345678901234567890123 4 5 */
    /* Mon Dec 24 15:44:14 1990\n\0 */

    date[0] = timestring[4];		/* form "Dec 24 1990" */
    date[1] = timestring[5];		/*       01234567890 */
    date[2] = timestring[6];
    date[3] = timestring[7];
    date[4] = timestring[8];
    date[5] = timestring[9];
    date[6] = timestring[10];
    date[7] = timestring[20];
    date[8] = timestring[21];
    date[9] = timestring[22];
    date[10] = timestring[23];
    date[11] = '\0';

    timestring[19] = '\0';

    dodef2("__DATE__",date,1);
    dodef2("__TIME__",&timestring[11],1);

    timestring[0] = '\0';

#if !defined(__TURBOC__)
#if STDC
#if defined(_AIX370) && defined(__cplusplus)
    /* prototype for strftime() in /usr/local/lib/CC-include/time.h is wrong */
    tzset();			/* set daylight and tzname externals */
    (void)strncpy(timestring,daylight ? tzname[1] : tzname[0],3);
#else
    (void)strftime(timestring,4,"%Z",localtime(&thetimeval));
#endif /* defined(_AIX370) && defined(__cplusplus) */
#else /* NOT STDC */
#ifdef ardent
    /* tm_zone is not available */
#else
#if defined(__hpux) || defined(_IBMR2)
    tzset();			/* set daylight and tzname externals */
    (void)strncpy(timestring,daylight ? tzname[1] : tzname[0],3);
#else
    (void)strncpy(timestring,localtime(&thetimeval)->tm_zone,3);
#endif /* defined(__hpux) || defined(_IBMR2) */
#endif /* ardent */
#endif /* STDC */
#endif /* !defined(__TURBOC__) */

    timestring[3] = '\0';
    dodef2("__TIMEZONE__",timestring,1);

    p = getcwd(fpp_line,sizeof(fpp_line));
    if (p == (char*)NULL)
	p = getenv("cwd");
    PRINTF("%s Directory: %s\n",OUT_PREFIX,p);

#if    unix
    /* output Jill Q. Username <user_name@machine.site.org> */
    user_name = getlogin();
    if ( (user_name == (const char*)NULL) || (user_name[0] == '\0') )
	user_name = (const char*)Cuserid((char*)NULL);
    passwdp = (struct passwd*)getpwuid(getuid());
    if (passwdp != (struct passwd *)NULL)
    {
	if ( (user_name == (const char*)NULL) || (user_name[0] == '\0') )
	    user_name = passwdp->pw_name;
	if (passwdp->pw_gecos[0] == '&')
	    personal_name = user_name;		/* & means use login name */
	else
	    personal_name = passwdp->pw_gecos;	/* have personal name */

#if __convex__ || __sgi
	/* Convex and SGI store "John Doe, office, phone" in
	   /etc/passwd entry */
	if (personal_name != (char*)NULL)
	{
	    p = strchr(personal_name,',');
	    if (p != (char*)NULL)
	        *p = '\0';
	}
#endif

    }
    if ( (user_name == (const char*)NULL) || (user_name[0] == '\0') )
	user_name = (const char*)getenv("USER");
    if ( (user_name == (const char*)NULL) || (user_name[0] == '\0') )
        user_name = "";
    if (gethostname(hostname,sizeof(hostname)) == 0)
	hostname[sizeof(hostname)-1] = '\0'; /* ensure final NULL */
    else
    {
	p = getenv("HOST");
	if (p != (char*)NULL)
	    p = getenv("HOSTNAME");
	if (p != (char*)NULL)
	{
	    (void)strncpy(hostname,p,sizeof(hostname));
	    hostname[sizeof(hostname)-1] = '\0'; /* ensure final NULL */
	}
	else				/* no hostname available */
	    hostname[0] = '\0';
    }

    PRINTF("%s User: %s <%s%s%s>\n",
	OUT_PREFIX,personal_name,user_name,hostname[0] ? "@" : "",hostname);
#endif /* unix */

}
